@@ -104,6 +104,9 @@ or a function symbol which, when called, returns such a cons cell."
   (interactive "zkeyboard-coding-system: ")
   (get-coding-system coding-system) ; correctness check
   (setq keyboard-coding-system coding-system)
+  (if (eq (device-type) 'tty)
+      (set-console-tty-input-coding-system
+       (device-console) keyboard-coding-system))
   (redraw-modeline t))
 
 (defsubst terminal-coding-system ()
@@ -117,7 +120,8 @@ or a function symbol which, when called, returns such a cons cell."
   (setq terminal-coding-system coding-system)
   ; #### should this affect all current tty consoles ?
   (if (eq (device-type) 'tty)
-      (set-console-tty-coding-system (device-console) terminal-coding-system))
+      (set-console-tty-output-coding-system
+       (device-console) terminal-coding-system))
   (redraw-modeline t))
 
 (defun set-pathname-coding-system (coding-system)
