@@ -15,7 +15,7 @@
 (custom-add-loads 'menu '("menubar-items"))
 (custom-add-loads 'minibuffer '("simple" "minibuf"))
 (custom-add-loads 'log-message '("simple"))
-(custom-add-loads 'environment '("frame" "keydefs" "minibuf" "modeline" "window-xemacs" "menubar" "gutter-items" "x-init" "toolbar-items" "cus-edit" "gnuserv" "sound"))
+(custom-add-loads 'environment '("frame" "keydefs" "minibuf" "modeline" "window-xemacs" "menubar" "gutter" "toolbar-items" "cus-edit" "gnuserv" "sound" "x-init"))
 (custom-add-loads 'sound '("sound"))
 (custom-add-loads 'pui '("package-ui"))
 (custom-add-loads 'terminals '("gnuserv"))
@@ -47,6 +47,7 @@
 (custom-add-loads 'custom-browse '("cus-edit"))
 (custom-add-loads 'abbrev '("abbrev" "files"))
 (custom-add-loads 'programming '("cus-edit"))
+(custom-add-loads 'printing '("printer"))
 (custom-add-loads 'toolbar '("toolbar-items"))
 (custom-add-loads 'dired '("files"))
 (custom-add-loads 'dnd-debug '("dragdrop"))
@@ -62,7 +63,7 @@
 (custom-add-loads 'external '("process" "cus-edit"))
 (custom-add-loads 'development '("process" "lisp-mode" "cus-edit"))
 (custom-add-loads 'gnuserv '("gnuserv"))
-(custom-add-loads 'gutter '("gutter-items"))
+(custom-add-loads 'gutter '("gutter" "gutter-items"))
 (custom-add-loads 'fill-comments '("simple"))
 (custom-add-loads 'windows '("window" "window-xemacs"))
 (custom-add-loads 'widget-faces '("wid-edit"))
@@ -70,16 +71,16 @@
 (custom-add-loads 'fill '("simple" "fill"))
 (custom-add-loads 'custom-magic-faces '("cus-edit"))
 (custom-add-loads 'display '("toolbar" "scrollbar" "auto-show"))
-(custom-add-loads 'faces '("faces" "cus-edit" "font-lock" "font" "hyper-apropos" "info" "wid-edit"))
+(custom-add-loads 'faces '("faces" "font" "cus-edit" "font-lock" "hyper-apropos" "info" "wid-edit"))
 (custom-add-loads 'emacs '("faces" "help" "files" "cus-edit" "package-get"))
 (custom-add-loads 'processes '("process" "gnuserv"))
 (custom-add-loads 'hyper-apropos '("hyper-apropos"))
-(custom-add-loads 'wp '("cus-edit"))
+(custom-add-loads 'wp '("printer" "cus-edit"))
 (custom-add-loads 'vc '("files"))
 (custom-add-loads 'isearch '("isearch-mode"))
 (custom-add-loads 'font-lock-faces '("font-lock"))
 (custom-add-loads 'modeline '("modeline"))
-(custom-add-loads 'editing '("simple" "abbrev" "fill" "mouse" "cus-edit" "dragdrop"))
+(custom-add-loads 'editing '("simple" "abbrev" "fill" "mouse" "dragdrop" "cus-edit"))
 (custom-add-loads 'matching '("simple" "isearch-mode" "hyper-apropos"))
 (custom-add-loads 'i18n '("cus-edit"))
 (custom-add-loads 'info '("toolbar-items" "info"))
