@@ -66,6 +66,7 @@ Lisp_Object Qdefault;
 Lisp_Object Qdelete;
 Lisp_Object Qdelq;
 Lisp_Object Qdevice;
+Lisp_Object Qdialog;
 Lisp_Object Qdimension;
 Lisp_Object Qdisplay;
 Lisp_Object Qdoc_string;
@@ -82,6 +83,7 @@ Lisp_Object Qfile_name;
 Lisp_Object Qfont;
 Lisp_Object Qframe;
 Lisp_Object Qfunction;
+Lisp_Object Qfuncall;
 Lisp_Object Qgap_overhead;
 Lisp_Object Qgeneric;
 Lisp_Object Qgeometry;
@@ -102,14 +104,17 @@ Lisp_Object Qkey_assoc;
 Lisp_Object Qkeyboard;
 Lisp_Object Qkeymap;
 Lisp_Object Qlandscape;
+Lisp_Object Qlast_command;
 Lisp_Object Qleft;
 Lisp_Object Qleft_margin;
+Lisp_Object Qlet;
 Lisp_Object Qlist;
 Lisp_Object Qmagic;
 Lisp_Object Qmalloc_overhead;
 Lisp_Object Qmarkers;
 Lisp_Object Qmax;
 Lisp_Object Qmemory;
+Lisp_Object Qmenubar;
 Lisp_Object Qmessage;
 Lisp_Object Qminus;
 Lisp_Object Qmodifiers;
@@ -160,6 +165,7 @@ Lisp_Object Qsyntax;
 Lisp_Object Qterminal;
 Lisp_Object Qtest;
 Lisp_Object Qtext;
+Lisp_Object Qthis_command;
 Lisp_Object Qtimeout;
 Lisp_Object Qtimestamp;
 Lisp_Object Qtoolbar;
@@ -218,6 +224,7 @@ syms_of_general (void)
   defsymbol (&Qdelete, "delete");
   defsymbol (&Qdelq, "delq");
   defsymbol (&Qdevice, "device");
+  defsymbol (&Qdialog, "dialog");
   defsymbol (&Qdimension, "dimension");
   defsymbol (&Qdisplay, "display");
   defsymbol (&Qdoc_string, "doc-string");
@@ -234,6 +241,7 @@ syms_of_general (void)
   defsymbol (&Qfont, "font");
   defsymbol (&Qframe, "frame");
   defsymbol (&Qfunction, "function");
+  defsymbol (&Qfuncall, "funcall");
   defsymbol (&Qgap_overhead, "gap-overhead");
   defsymbol (&Qgeneric, "generic");
   defsymbol (&Qgeometry, "geometry");
@@ -254,14 +262,17 @@ syms_of_general (void)
   defsymbol (&Qkeyboard, "keyboard");
   defsymbol (&Qkeymap, "keymap");
   defsymbol (&Qlandscape, "landscape");
+  defsymbol (&Qlast_command, "last-command");
   defsymbol (&Qleft, "left");
   defsymbol (&Qleft_margin, "left-margin");
+  defsymbol (&Qlet, "let");
   defsymbol (&Qlist, "list");
   defsymbol (&Qmagic, "magic");
   defsymbol (&Qmalloc_overhead, "malloc-overhead");
   defsymbol (&Qmarkers, "markers");
   defsymbol (&Qmax, "max");
   defsymbol (&Qmemory, "memory");
+  defsymbol (&Qmenubar, "menubar");
   defsymbol (&Qmessage, "message");
   defsymbol (&Qminus, "-");
   defsymbol (&Qmodifiers, "modifiers");
@@ -312,6 +323,7 @@ syms_of_general (void)
   defsymbol (&Qterminal, "terminal");
   defsymbol (&Qtest, "test");
   defsymbol (&Qtext, "text");
+  defsymbol (&Qthis_command, "this-command");
   defsymbol (&Qtimeout, "timeout");
   defsymbol (&Qtimestamp, "timestamp");
   defsymbol (&Qtoolbar, "toolbar");
