@@ -20,18 +20,6 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: FSF 19.31. */
 
-/* Define WORD_MACHINE if addresses and such have
- * to be corrected before they can be used as byte counts.  */
-
-#define WORD_MACHINE
-
-/* Define DATA_SEG_BITS if pointers need to be corrected with
-   a segment field.  */
-
-#ifdef FIX_ADDRESS
-#define DATA_SEG_BITS 0xef000000
-#endif
-
 /* Now define a symbol for the cpu type, if your compiler
    does not define it automatically:
    Ones defined so far include vax, m68000, ns16000, pyramid,
@@ -41,14 +29,6 @@ Boston, MA 02111-1307, USA.  */
 /*#define m88k*/
 /*#endif*/
 
-/* Define EXPLICIT_SIGN_EXTEND if XINT must explicitly sign-extend
-   the bit field into an int.  In other words, if bit fields
-   are always unsigned.
-
-   If you use NO_UNION_TYPE, this flag does not matter.  */
-
-#define EXPLICIT_SIGN_EXTEND
-
 /* Data type of load average, as read out of kmem.  */
 
 /* #define LOAD_AVE_TYPE long */
@@ -100,10 +80,6 @@ Boston, MA 02111-1307, USA.  */
 
 #define	ADDR_CORRECT(ADDR) ((int)ADDR)
 
-/* Cast pointers to this type to compare them. */
-
-#define	PNTR_COMPARISON_TYPE	void *
-
 /* Some machines that use COFF executables require that each section
    start on a certain boundary *in the COFF file*.  Such machines should
    define SECTION_ALIGNMENT to a mask of the low-order bits that must be
