@@ -336,59 +336,6 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 		{
 		  switch (XIMAGE_INSTANCE_TYPE (instance))
 		    {
-		    case IMAGE_TEXT:
-		      {
-			Bufbyte *temptemp;
-			Lisp_Object string =
-			  XIMAGE_INSTANCE_TEXT_STRING (instance);
-			Bytecount len = XSTRING_LENGTH (string);
-			
-			/* In the unlikely instance that a garbage-collect
-			   occurs during encoding, we at least need to
-			   copy the string.
-			*/
-			temptemp = (Bufbyte *) alloca (len);
-			memcpy (temptemp, XSTRING_DATA (string), len);
-			{
-			  int i;
-			  
-			  /* Now truncate the first rb->object.dglyph.xoffset
-			     columns. */
-			  for (i = 0; i < rb->object.dglyph.xoffset;)
-			    {
-#ifdef MULE
-			      Emchar ch = charptr_emchar (temptemp);
-			      i += XCHARSET_COLUMNS (CHAR_CHARSET (ch));
-#else
-			      i++; /* telescope this */
-#endif
-			      INC_CHARPTR (temptemp);
-			    }
-			  
-			  /* If we truncated one column too many, then
-			     add a space at the beginning. */
-			  if (i > rb->object.dglyph.xoffset)
-			    {
-			      assert (i > 0);
-			      *--temptemp = ' ';
-			      i--;
-			    }
-			  len -= i;
-			}
-			
-			tty_output_bufbyte_string (w, dl, temptemp, len,
-						   xpos, findex, 0);
-			
-			if (xpos >= cursor_start
-			    && (cursor_start <
-				xpos + (bufbyte_string_displayed_columns
-					(temptemp, len))))
-			  {
-			    cmgoto (f, dl->ypos - 1, cursor_start);
-			  }
-		      }
-		      break;
-		      
 		    case IMAGE_MONO_PIXMAP:
 		    case IMAGE_COLOR_PIXMAP:
 		    case IMAGE_SUBWINDOW:
@@ -397,13 +344,12 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 		      /* just do nothing here */
 		      break;
 		      
-		    case IMAGE_POINTER:
-		      abort ();
-		      
 		    case IMAGE_NOTHING:
 		      /* nothing is as nothing does */
 		      break;
 		      
+		    case IMAGE_TEXT:
+		    case IMAGE_POINTER:
 		    default:
 		      abort ();
 		    }
