@@ -67,7 +67,7 @@ Boston, MA 02111-1307, USA.  */
 /* encapsulation: directory calls */
 
 #ifdef ENCAPSULATE_CHDIR
-int sys_chdir (CONST char *path);
+int sys_chdir (const char *path);
 #endif
 #if defined (ENCAPSULATE_CHDIR) && !defined (DONT_ENCAPSULATE)
 # undef chdir
@@ -78,7 +78,7 @@ int sys_chdir (CONST char *path);
 #endif
 
 #ifdef ENCAPSULATE_MKDIR
-int sys_mkdir (CONST char *path, mode_t mode);
+int sys_mkdir (const char *path, mode_t mode);
 #endif
 #if defined (ENCAPSULATE_MKDIR) && !defined (DONT_ENCAPSULATE)
 # undef mkdir
@@ -89,7 +89,7 @@ int sys_mkdir (CONST char *path, mode_t mode);
 #endif
 
 #ifdef ENCAPSULATE_OPENDIR
-DIR *sys_opendir (CONST char *filename);
+DIR *sys_opendir (const char *filename);
 #endif
 #if defined (ENCAPSULATE_OPENDIR) && !defined (DONT_ENCAPSULATE)
 # undef opendir
@@ -122,7 +122,7 @@ int sys_closedir (DIR *dirp);
 #endif
 
 #ifdef ENCAPSULATE_RMDIR
-int sys_rmdir (CONST char *path);
+int sys_rmdir (const char *path);
 #endif
 #if defined (ENCAPSULATE_RMDIR) && !defined (DONT_ENCAPSULATE)
 # undef rmdir
