@@ -2,8 +2,8 @@
 emacs_is_beta=t
 emacs_major_version=21
 emacs_minor_version=2
-emacs_beta_version=32
-xemacs_codename="Kastor & Polydeukes"
+emacs_beta_version=33
+xemacs_codename="Melpomene"
 infodock_major_version=4
 infodock_minor_version=0
 infodock_build_version=8
